using System;
using Server;
using Server.Mobiles;
using Server.Network;
using Server.Gumps;
using System.Collections.Generic;
namespace Server
{
	public static class UnicodeAnimator  
	{
		public  static void PlayAnimationOnLocation( List<string[]>  AnimationSequence, List<int>  AnimationSequenceHues, Point3D loc, Map map, double fps ){
			loc.Y = loc.Y+1;
			loc.X = loc.X+1;			
			PlaySingleAnimation(AnimationSequence, AnimationSequenceHues, null, loc, map, (int)(1000/fps));
		}	

		public  static void PlayAnimationOnMobile( List<string[]>  AnimationSequence, List<int>  AnimationSequenceHues, Mobile m, double fps ){
		
			PlaySingleAnimationOnMobile(AnimationSequence, AnimationSequenceHues, null,m, (int)(1000/fps));
		}
		/**
		*
		*This function basically just plays a single custome animation, frame by frame, by calling itself recursivly
		*
		**/
		private  static void PlaySingleAnimation( List<string[]>  AnimationSequence,List<int>  AnimationSequenceHues, Item i, Point3D loc, Map map,int  fps ){		
			//we have to do this with a callback, to not delay anything else
			TimerCallback callback = new TimerCallback( delegate( ) {
				//delete item if its there
				if(i != null){
					i.Delete();
					i=null;
				}
				//recreate a no-draw item (clears text for next animation), so it works!
				if(i == null){
					i = new Item( 0x0001); //invisible item to client
					i.MoveToWorld(loc, map);
				}
				if(AnimationSequence.Count > 0){
					SendAnimatePacket( i,  MessageType.Regular, (int)AnimationSequenceHues[0],   (string[])AnimationSequence[0] ,6, loc, map);
					AnimationSequence.RemoveAt(0);
					AnimationSequenceHues.RemoveAt(0);
					PlaySingleAnimation( AnimationSequence,AnimationSequenceHues, i, loc, map, fps);
				}else{
					i.Delete();
					i = null;
					return;
				}
			});	
			Timer timer = Timer.DelayCall( TimeSpan.FromMilliseconds(fps), callback	);		
		}
		/**
		* Animates around a mobile, for effects and such you want to broadcast from spells and such
		*/
		private  static void PlaySingleAnimationOnMobile( List<string[]>  AnimationSequence,List<int>  AnimationSequenceHues, Item i, Mobile m, int  fps ){					//we have to do this with a callback, to not delay anything else
			TimerCallback callback = new TimerCallback( delegate( ) {
				Point3D loc = m.Location;
				//make sure their alive
				if( !m.Alive){
					return;
				}
				//delete item if its there
				if(i != null){
					i.Delete();
					i=null;
				}
				//recreate a no-draw item (clears text for next animation), so it works!
				if(i == null){
					i = new Item( 0x0001); //invisible item to client
					loc.Y = loc.Y+1;
					loc.X = loc.X+1;	
					i.MoveToWorld(loc, m.Map);
				}
				//if animation count
				if(AnimationSequence.Count > 0){	
					SendAnimatePacket( i,  MessageType.Regular    , (int)AnimationSequenceHues[0],   (string[])AnimationSequence[0] , 6, loc, m.Map);
					AnimationSequence.RemoveAt(0);
					AnimationSequenceHues.RemoveAt(0);
					PlaySingleAnimationOnMobile( AnimationSequence,AnimationSequenceHues, i, m, fps);
				}else{
					i.Delete();
					i = null;
					return;
				}
			});	
			Timer timer = Timer.DelayCall( TimeSpan.FromMilliseconds(fps), callback	);		
		}
		private static void SendAnimatePacket( Item i, MessageType type, int hue, string[] text, int fontType, Point3D loc, Map map){
			if ( map != null )
			{			
				Packet p = null;
				IPooledEnumerable eable = i.Map.GetClientsInRange( (Point3D) loc );
				string outputStr = "";
				foreach (string t in text){
					outputStr = outputStr + t +"\n";
				}
				foreach ( NetState state in eable )	{
					if ( state.Mobile.CanSee(i)  )	{
						if(p == null && hue != null){
							p = new UnicodeMessage( i.Serial, (int)MobileDelta.None, type, hue, fontType, "ENU", "", outputStr );
							p.Acquire();
						}
						state.Send( p );
					}
				}

				Packet.Release( p );

				eable.Free();
			}
		}
	}
}